<?php

namespace Rokt\Plugin\Ecommerce\Admin;

defined( 'ABSPATH' ) || exit;

use Rokt\Plugin\Ecommerce\Includes\Constants;
use Rokt\Plugin\Ecommerce\Includes\Helper;
use Rokt\Plugin\Ecommerce\Includes\PluginHelper;
use Rokt\Plugin\Ecommerce\Includes\RoktApiClient;
use Rokt\Plugin\Ecommerce\Plugin;

class Admin {

	public $plugin;
	private $rokt_api_client;

	public function __construct( $plugin ) {
		$this->plugin          = $plugin;
		$this->rokt_api_client = new RoktApiClient($plugin->config);
	}

	public function enqueue_styles( $hook ) {
		$style_path            = '/build/app.css';
		$wp_style_dependencies = array( 'wp-components', 'wc-components' );

		wp_register_style(
			Plugin::$plugin_name,
			plugins_url( $style_path, ROKT_ECOMMERCE_PLUGIN_FILE ),
			$wp_style_dependencies,
			filemtime( untrailingslashit( plugin_dir_path( ROKT_ECOMMERCE_PLUGIN_FILE ) ) . $style_path )
		);

		wp_enqueue_style( Plugin::$plugin_name );
	}

	public function enqueue_scripts( $hook ) {
		$script_path       = '/build/app.js';
		$script_asset_path = plugin_dir_path( ROKT_ECOMMERCE_PLUGIN_FILE ) . 'build/app.asset.php';

		$script_asset = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array( 'wp-element', 'wp-components', 'wp-i18n', 'wc-components', 'wc-date' ),
				'version'      => filemtime( untrailingslashit( plugin_dir_path( ROKT_ECOMMERCE_PLUGIN_FILE ) ) . $script_path ),
			);

		$script_url = plugins_url( $script_path, ROKT_ECOMMERCE_PLUGIN_FILE );

		wp_register_script(
			Plugin::$plugin_name,
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);

		wp_add_inline_script(
			Plugin::$plugin_name,
			'var roktApiSettings=' . wp_json_encode(
				array(
					'baseUrl' => esc_url_raw( rest_url() ),
					'nonce'   => wp_create_nonce( 'wp_rest' ),
				)
			),
			'before'
		);

		wp_enqueue_script( Plugin::$plugin_name );

		wp_set_script_translations( Plugin::$plugin_name, Plugin::$plugin_name, ROKT_ECOMMERCE_ABSPATH . '/languages' );
	}

	public function inject_before_notices() {
		if ( ! $this->is_rokt_admin_page() ) {
			return;
		}
		// Wrap the notices in a hidden div to prevent flickering before
		// they are moved elsewhere in the page by WordPress Core.
		echo '<div class="rokt-ecommerce__notice-list-hide" style="display:none;">';

		// Capture all notices and hide them. WordPress Core looks for
		// `.wp-header-end` and appends notices after it if found.
		// https://github.com/WordPress/WordPress/blob/f6a37e7d39e2534d05b9e542045174498edfe536/wp-admin/js/common.js#L737 .
		echo '<div class="wp-header-end"></div>';
	}

	public function get_started_notice() {
		if ( ! PluginHelper::check_woocommerce_plugin_activated() ) {
			return;
		}

		if ( $this->is_configured() ) {
			return;
		}

		return include_once dirname( __FILE__ ) . '/partials/get-started-notice.php';
	}

	public function inject_after_notices() {
		if ( ! $this->is_rokt_admin_page() ) {
			return;
		}
		// Close the hidden div used to prevent notices from flickering before
		// they are inserted elsewhere in the page.
		echo '</div>';
	}


	public static function required_woocommerce_activation_notice() {
		add_action(
			'admin_notices',
			function() {
				include_once dirname( __FILE__ ) . '/partials/woocommerce-status-notice.php';
			},
			10
		);
	}

	public function is_rokt_admin_page() {
		global $pagenow;

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( 'admin.php' === $pagenow && isset( $_GET['page'] ) && Plugin::$plugin_name === $_GET['page'] ) {
			return true;
		}
		return false;
	}

	private function is_configured() {
		$has_api_key = ! empty( Helper::get_option( Constants::API_SECRET_KEY ) );
		return $has_api_key;
	}

	public function add_plugin_admin_menu() {
		add_submenu_page(
			'woocommerce',
			'Rokt Ecommerce',
			'Rokt Ecommerce',
			Plugin::$required_capability,
			Plugin::$plugin_name,
			array( $this, 'render_app_placeholder' ),
			20
		);

		$this->remove_wp_admin_default_footer();
	}

	public function listen_shop_update() {
		$this->listen_on_blogname_update();
		$this->listen_on_home_url_update();
		$this->listen_on_admin_email_update();
	}

	private function listen_on_blogname_update() {
		add_action('update_option_blogname', function ( $old_value, $new_value) {
			$this->sending_shop_details(
				array(
					'brand' => $new_value
				)
			);
		}, 10, 3 );
	}

	private function listen_on_home_url_update() {
		add_action('update_option_home', function ( $old_value, $new_value) {
			$this->sending_shop_details(
				array(
					'websiteUrl' => $new_value
				)
			);
		}, 10, 3 );
	}

	private function listen_on_admin_email_update() {
		add_action('update_option_admin_email', function ( $old_value, $new_admin_email) {
			$user            = get_user_by( 'email', $new_admin_email );
			$new_user_detail = array();
			if ( $user ) {
				$new_user_detail = Helper::get_account_name($user);
			}
			$this->sending_shop_details(
				array_merge(
				array(
					'primaryEmail' => $new_admin_email
				),
				$new_user_detail)
			);
		}, 10, 3 );
	}

	private function remove_wp_admin_default_footer() {
		add_filter( 'admin_footer_text', '__return_empty_string', 11 );
		add_filter( 'update_footer', '__return_empty_string', 11 );
	}

	private function get_app_versions() {
		$body         = $this->rokt_api_client->request('/v1/info');
		$app_versions = ! empty( $body ) ? $body['apps']['woocommerce'] : [];
		return $app_versions;
	}

	private function sending_shop_details( $data = [] ) {
		$shop_update_endpoint = '/v1/appinstallations/shop-update';
		$args                 = array (
			'method'=> \WP_REST_Server::CREATABLE,
			'body' =>  wp_json_encode ( $data )
			);
		try {
			$this->rokt_api_client->request($shop_update_endpoint , $args );
		} catch (\Exception $e) {
			$this->rokt_api_client->logError($shop_update_endpoint, $e->getMessage());
		}
	}

	public function render_app_placeholder() {
		$user_api_key      = Helper::get_option( Constants::API_SECRET_KEY, '' );
		$wp_user           = wp_get_current_user();
		$rokt_api_endpoint = $this->plugin->config['rokt_api_endpoint_for_frontend'];
		$blog_url          = get_bloginfo( 'url' );

		$initial_props          = new \stdClass();
		$initial_props->wp_user = [
			'email'     => $wp_user->user_email,
			'firstName' => $wp_user->user_firstname,
			'lastName'  => $wp_user->user_lastname,
			'name'      => $wp_user->display_name,
		];

		$initial_props->rokt_account         = Helper::get_rokt_account();
		$initial_props->is_authenticated     = ! empty( $user_api_key );
		$initial_props->plugin_url           = admin_url( 'admin.php?page=' . Plugin::$plugin_name );
		$initial_props->plugin_directory_url = plugins_url( '', ROKT_ECOMMERCE_PLUGIN_FILE );
		$initial_props->site_rest_url        = get_rest_url();
		$initial_props->rokt_api_endpoint    = $rokt_api_endpoint;
		$initial_props->app_versions         = $this->get_app_versions();
		$initial_props->base_location        = wc_get_base_location();
		$initial_props->site_title           = get_bloginfo( 'name' );
		$initial_props->website_url          = ! empty( $blog_url ) ? $blog_url : get_bloginfo( 'wpurl' );

		include_once dirname( __FILE__ ) . '/partials/main-page.php';
	}

}
